/**
 * Level.java
 * 2/9/11
 */

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import javax.swing.JPanel;
import javax.swing.Timer;

/**
 * Platform game level.
 * 
 * @author CPSC170A
 */
public class Level extends JPanel implements ActionListener {

    /** Prevent eclipse warning. */
    private static final long serialVersionUID = 1L;

    /** Platform sprites. */
    private PlatformList platforms;

    /** Player sprite. */
    private Player player;

    /** Background sprite. */
    private Background background;

    /** Keyboard state. */
    private KeyboardState keyboardState;

    /** Viewport location. */
    private Point2D.Double viewport;

    /**
     * Constructor, initializes all sprites.
     */
    public Level() {
        Timer timer = new Timer(Settings.REFRESH, this);
        player = new Player(Settings.INIT_PLAYER_X, Settings.INIT_PLAYER_Y,
                Settings.PLAYER_WIDTH, Settings.PLAYER_HEIGHT);
        platforms = new PlatformList("platforms.txt");
        background = new Background(0.0, Settings.PANEL_HEIGHT
                - Settings.BG_HEIGHT, Settings.BG_WIDTH, Settings.BG_HEIGHT);
        keyboardState = new KeyboardState();
        viewport = new Point2D.Double(0.0, 0.0);
        addKeyListener(keyboardState);
        setBackground(Settings.BG_COLOR);
        setFocusable(true);
        setPreferredSize(new Dimension(Settings.PANEL_WIDTH,
                Settings.PANEL_HEIGHT));
        timer.start();
    }

    /**
     * Draw the level.
     * 
     * @param g graphics object to draw to
     */
    public final void paintComponent(final Graphics g) {
        super.paintComponent(g);
        background.paint(g, viewport);
        platforms.paint(g, viewport);
        player.paint(g, viewport);
    }

    /**
     * Update the the game every frame of animation.
     * 
     * @param event timer event
     */
    public final void actionPerformed(final ActionEvent event) {
        player.update(keyboardState, platforms);
        updateViewport();
        repaint();
    }

    /**
     * Update the location of the viewport based on the player location and
     * velocity.
     */
    private void updateViewport() {
        double localX = player.x - viewport.x;
        double localY = player.y - viewport.y;
        double rightBound = Settings.RIGHT_BOUND;
        double leftBound = Settings.LEFT_BOUND;
        double topBound = Settings.TOP_BOUND;
        double bottomBound = Settings.BOTTOM_BOUND;
        if (player.deltaX > 0.0 && localX + player.width > rightBound) {
            viewport.x += ((localX + player.width) - rightBound);
        }
        if (player.deltaX < 0.0 && localX < leftBound) {
            viewport.x -= (leftBound - (localX + player.width));
        }
        if (player.deltaY < 0.0 && localY < topBound) {
            viewport.y -= (topBound - localY);
        }
        if (player.deltaY > 0.0 && (localY + player.height) > bottomBound) {
            viewport.y += ((localY + player.height) - bottomBound);
        }
    }

}
